



bash -euf -c " hub version | grep -q 'hub version 2.13.0'; etc_conda=$PREFIX/etc/conda; for F in activate deactivate; do test -r $etc_conda/$F.d/hub.sh; test -f $etc_conda/$F.d/hub.ps1; done; test -r $LIBRARY_PREFIX/etc/bash_completion.d/hub.sh; "
IF %ERRORLEVEL% NEQ 0 exit /B 1
exit /B 0
